#include "tiletype.h"

#include "sprite.h"
#include "tileset.h"
#include "world.h"

#include <cstdlib> // rand().

class TileType * TileType::Self = (class TileType *) 1;

TileType::TileType(
	unsigned short int aspect,
	std::string name,
	bool passable,
	unsigned short int growpower,
	class TileType * output,
	unsigned short int timer,
	enum Pattern pattern
) :
	aspect(aspect),
	name(name),
	passable(passable),
	growpower(growpower),
	output(output),
	timer(timer),
	pattern(pattern),
	death_spirit(false)
{
	if(this->output == TileType::Self) {
		this->output = this;
	}
}

TileType::~TileType() { }

void TileType::set_output(class TileType * t) {
	this->output = t;
}

std::string TileType::get_name() {
	return(this->name);
}

bool TileType::is_passable() {
	return(this->passable);
}

unsigned short int TileType::get_aspect() {
	return(this->aspect);
}

class Sprite * TileType::get_aspect(class Tileset * tileset) {
	return(tileset->get(this->aspect));
}

unsigned short int TileType::get_timer() {
	return(this->timer);
}

unsigned short int TileType::get_growpower() {
	return(this->growpower);
}

class TileType * TileType::get_output() {
	return(this->output);
}

enum Pattern TileType::get_pattern() {
	return(this->pattern);
}

void TileType::flag_as_death_spirit() {
	this->death_spirit = true;
}

bool TileType::is_death_spirit() {
	return(this->death_spirit);
}

// Try to overgrow on x-y.
void TileType::grow(
	class World * world,
	unsigned int x,
	unsigned int y
) {
	class TileType * old = world->get(x,y);
	if(
		old != nullptr
		and old->get_growpower() < this->get_growpower()
	) {
		world->set(x,y,this);
	}
}

// Grow around x-y accordingly to pattern.
void TileType::grow(
	class World * world,
	unsigned int x,
	unsigned int y,
	enum Pattern pattern
) {
	if(pattern == Pattern::Grass) { // Grass randomly grows on adjacent tiles.
		unsigned int dice = rand()%5;
		if(dice == 1 && y > 0) {
			this->grow(world, x, y-1);
		} else if(dice == 2 && y < world->get_height()-1) {
			this->grow(world, x, y+1);
		} else if(dice == 3 && x > 0) {
			this->grow(world, x-1, y);
		} else if(dice == 4 && x < world->get_width()-1) {
			this->grow(world, x+1, y);
		}
	} else if(pattern == Pattern::Spores) { // Spores grow like grass but can reach further.
		int dice_x = (rand()%5)-2;
		int dice_y = (rand()%5)-2;
		int X = x;
		int Y = y;
		if(X >= dice_x and X + dice_x < world->get_width()-1
			and Y >= dice_y and Y + dice_y < world->get_height()-1
			and not ((dice_x == -2 or dice_x == 2) and (dice_y == -2 or dice_y == 2)) // (Don't grow in corners.)
		) {
			this->grow(world, X + dice_x, Y + dice_y);
		}
	} else if(pattern == Pattern::VineVertical) { // Vines grow in two directions.
		if(y > 0) {
			this->grow(world, x, y-1);
		}
		if(y < world->get_height()-1) {
			this->grow(world, x, y+1);
		}
	} else if(pattern == Pattern::VineHorizontal) { // Vines grow in two directions.
		if(x > 0) {
			this->grow(world, x-1, y);
		}
		if(x < world->get_width()-1) {
			this->grow(world, x+1, y);
		}
	} else if(pattern == Pattern::Plague) { // Grows like grass and suicide after a while.
		// TODO
	} else if(pattern == Pattern::AlgaeWater) { // Algae grows only on water.
		// TODO
	}
}

