#ifndef TILETYPE_H
#define TILETYPE_H

#include <string>

class Sprite;
class Tileset;
class World;

#include "pattern.h"

class TileType {
	private:
		unsigned short int aspect; // Identifier in a tileset.
		std::string name;
		bool passable;
		unsigned short int growpower;
		class TileType * output;
		unsigned short int timer;
		enum Pattern pattern;
		bool death_spirit;
	public:
		TileType(
			unsigned short int aspect,
			std::string name,
			bool passable,
			unsigned short int growpower = 0, // 0 is always overgrown and never overgrows.
			class TileType * output = nullptr,
			unsigned short int timer = 0,     // 0 never ticks.
			enum Pattern pattern = Pattern::Passive);
		~TileType();
		void set_output(class TileType * t);
		std::string get_name();
		bool is_passable();
		unsigned short int get_aspect();
		class Sprite * get_aspect(class Tileset * tileset);
		unsigned short int get_timer();
		unsigned short int get_growpower();
		class TileType * get_output();
		enum Pattern get_pattern();
		void flag_as_death_spirit();
		bool is_death_spirit();
		void grow(class World * world, unsigned int x, unsigned int y); // Try to overgrow on x-y.
		void grow(class World * world, unsigned int x, unsigned int y, enum Pattern pattern); // Grow around x-y accordingly to pattern.

		static class TileType * Self;
};

#endif
