#include "window.h"

#include "sdl.h"
#include "tileset.h"
#include "sprite.h"
#include "world.h"
#include "tiletype.h"
#include "portal.h"

Window::Window(
	unsigned int width,
	unsigned int height,
	unsigned int tile_width,
	unsigned int tile_height,
	int x_shift,
	int y_shift,
	int x_center,
	int y_center
) :
	width(width),
	height(height),
	tile_width(tile_width),
	tile_height(tile_height),
	x_shift(x_shift),
	y_shift(y_shift),
	x_center(x_center),
	y_center(y_center)
{ }

void Window::set_center(int x_center, int y_center) {
	this->x_center = x_center;
	this->y_center = y_center;
}

void Window::draw(class Sdl * sdl, class World * world, class Tileset * ts) {
	int top = this->y_center - this->height/2;
	int lft = this->x_center - this->width/2;
	int bot = top + this->height;
	int rgt = lft + this->width;

	for(int y = top ; y < bot; y++) {
		for(int x = lft; x < rgt; x++) {
			if(x>=0 && x<world->get_width() && y>=0 && y<world->get_height()) {
				class Sprite * sprite = world->get(x,y)->get_aspect(ts);
				this->draw(sdl, sprite, x, y);
			}
		}
	}

	for(class Portal * portal : world->get_portals()) {
		signed int x = portal->get_x(); // XXX
		signed int y = portal->get_y(); // XXX
		if(
			x >= lft and
			x <= rgt and
			y >= top and
			y <= bot
		) {
			this->draw(sdl, ts->get(portal->get_aspect()), portal->get_x(), portal->get_y());
		}
	}
}

void Window::draw(class Sdl * sdl, class Sprite * sprite, int x, int y) {
	int top = this->y_center - this->height/2;
	int left = this->x_center - this->width/2;

	// TODO: don't draw if outside of the window's boundaries.

	sprite->draw(sdl,
		(x-left)*this->tile_width + this->x_shift,
		(y-top)*this->tile_height + this->y_shift
	);
}

void Window::draw(class Sdl * sdl, int id, class Tileset * ts, int x, int y) {
	this->draw(sdl, ts->get(id), x, y);
}

