#ifndef WORLD_H
#define WORLD_H

class TileType;
class Portal;
class Sdl;
class Tileset;
class Window;
class Sdl;
class Console;
class TextRenderer;

#include "pattern.h"

#include <vector>
#include <set>

class World {
	private:
		unsigned int width;  // tiles
		unsigned int height; // tiles
		std::vector<class TileType *> tiles;
		short int death_spirits;
		std::set<class Portal *> portals;

	public:
		World(
			unsigned int width,
			unsigned int height,
			class TileType * base_tile
		);
		~World();
		unsigned int get_width();
		unsigned int get_height();
		class TileType * get(unsigned int x, unsigned int y);
		void set(unsigned int x, unsigned int y, TileType * val);
		void add_portal(class Portal * p);
		void remove_portal(class Portal * p);
		class Portal * get_portal(unsigned int x, unsigned int y);
		std::vector<class Portal *> get_portals();
		void fill_all(TileType * val);
		void fill_square(TileType * val, unsigned int left, unsigned int top, unsigned int right, unsigned int bottom);
		void fill_circle(TileType * val, unsigned int x, unsigned int y, unsigned int radius);
		void grow(
			unsigned short int game_age,
			class Sdl * sdl,
			class Console * console,
			class TextRenderer * font
		);
		int get_death_spirit_count();

		static const int NoDeathSpirit = -1;
		static const int UnknownDeathSpirit = -2;
};

struct GrowSave {
	unsigned int x;
	unsigned int y;
	class TileType * output;
	enum Pattern pattern;
};

#endif
